import 'package:analyzer/dart/element/element.dart';
import 'package:like_sealed/like_sealed.dart';
import 'package:like_sealed_gen/src/description/class_description.dart';
import 'package:like_sealed_gen/src/util.dart';

class SealedSwitchContent {
  final LikeSealed config;
  final ClassDescription parentType;
  final List<ClassDescription> types;

  SealedSwitchContent(
    this.config,
    this.parentType,
    this.types,
  );

  String content() {
    String typeChecker = "";
    String caseFunction = "";
    for (var type in types) {
      final typeName = type.name;
      if (typeName == parentType.name) {
        typeChecker += "onDefault(type)";
        caseFunction +=
            "T onDefault($typeName ${typeName.firstToLower}) => throw UnimplementedError();";
      } else {
        String functionName = typeName;
        if (config.shortName) {
          functionName = functionName.removeIf(parentType.name);
        }
        typeChecker += "type is $typeName? on$functionName(type): ";
        caseFunction +=
            "T on$functionName($typeName ${functionName.firstToLower});";
      }
    }

    return """
    abstract class ${parentType.name}Switch<T> implements SealedSwitch<${parentType.name},T> {
      T switchCase(${parentType.name} type)=>$typeChecker;
      $caseFunction
    }
    """;
  }
}
