import 'package:analyzer/dart/element/element.dart';
import 'package:like_sealed/like_sealed.dart';
import 'package:like_sealed_gen/src/description/class_description.dart';
import 'package:like_sealed_gen/src/util.dart';

class SealedSwitchImplContent {
  final LikeSealed config;
  final ClassDescription parentType;
  final List<ClassDescription> types;

  SealedSwitchImplContent(
    this.config,
    this.parentType,
    this.types,
  );

  String content() {
    String typeChecker = "";
    String field = "";
    String fillConstructorParameter = "";
    List<String> constructorParameter = [];
    List<String> requiredConstructorSuper = [];
    String caseFunction = "";

    for (var type in types) {
      final typeName = type.name;
      String functionName = typeName;
      if (config.shortName) {
        functionName = functionName.removeIf(parentType.name);
      }
      final filedName = functionName.isEmpty
          ? parentType.name.firstToLower
          : functionName.firstToLower;

      constructorParameter.add("T Function($typeName)? ${filedName}");

      requiredConstructorSuper.add("_${filedName}=${filedName}");
      fillConstructorParameter += "this._${filedName},";
      field += "final T Function($typeName)?  _${filedName};";

      if (typeName == parentType.name) {
        typeChecker +=
            "(type is $typeName&& _${filedName} != null)? onDefault(type) : throw UnimplementedError()";

        caseFunction += "T onDefault($typeName value)=> _${filedName}!(value);";
      } else {
        typeChecker +=
            "(type is $typeName && _${filedName} != null) ? on$functionName(type) : ";

        caseFunction +=
            "T on$functionName($typeName value)=> _${filedName}!(value) ;";
      }
    }

    var requiredConstructorParameterString = "";
    for (var value in constructorParameter) {
      requiredConstructorParameterString += "$value,";
    }
    var constructorParameterString = "";
    for (var value in constructorParameter) {
      constructorParameterString += "required $value,";
    }
    return """
    class ${parentType.name}SwitchImpl<T> implements ${parentType.name}Switch<T> {
      ${field}
      
      ${parentType.name}SwitchImpl({${requiredConstructorParameterString}}): ${requiredConstructorSuper.join(",")};
      
      @Deprecated("use base constructor or required")
      ${parentType.name}SwitchImpl.fill($fillConstructorParameter);
      
       ${parentType.name}SwitchImpl.required({${constructorParameterString}}): ${requiredConstructorSuper.join(",")};
     
      T switchCase(${parentType.name} type)=>$typeChecker;
      
      $caseFunction
    }
    """;
  }
}
